@@ -191,7 +191,8 @@ udisks_linux_manager_bcache_new (UDisksDaemon *daemon)
  * Returns: A #UDisksDaemon. Do not free, the object is owned by @manager.
  */
 
-UDisksDaemon *udisks_linux_manager_bcache_get_daemon (UDisksLinuxManagerBcache* manager)
+UDisksDaemon *
+udisks_linux_manager_bcache_get_daemon (UDisksLinuxManagerBcache* manager)
 {
   g_return_val_if_fail (UDISKS_IS_LINUX_MANAGER_BCACHE (manager), NULL);
   return manager->daemon;
@@ -231,10 +232,10 @@ handle_bcache_create (UDisksManagerBcache    *object,
 
   UDisksObject *backing_dev_object = NULL;
   UDisksBlock *backing_dev_block = NULL;
-  const gchar *backing_dev_path = NULL;
+  gchar *backing_dev_path = NULL;
   UDisksObject *cache_dev_object = NULL;
   UDisksBlock *cache_dev_block = NULL;
-  const gchar *cache_dev_path = NULL;
+  gchar *cache_dev_path = NULL;
   gchar *bcache_name = NULL;
   gchar *bcache_file = NULL;
   UDisksObject *bcache_object = NULL;
@@ -316,7 +317,7 @@ handle_bcache_create (UDisksManagerBcache    *object,
   if (bcache_object == NULL)
     {
       g_prefix_error (&error,
-                      "Error waiting for bcache object after creating %s",
+                      "Error waiting for bcache object after creating '%s': ",
                       bcache_name);
       g_dbus_method_invocation_take_error (invocation, error);
       goto out;
@@ -326,11 +327,11 @@ handle_bcache_create (UDisksManagerBcache    *object,
                                                 invocation,
                                                 g_dbus_object_get_object_path (G_DBUS_OBJECT (bcache_object)));
 out:
-  g_free ((gchar *) backing_dev_path);
-  g_free ((gchar *) cache_dev_path);
+  g_free (backing_dev_path);
+  g_free (cache_dev_path);
   g_free (bcache_name);
   g_free (bcache_file);
-  g_object_unref (bcache_object);
+  g_clear_object (&bcache_object);
   g_clear_object (&backing_dev_object);
   g_clear_object (&backing_dev_block);
   g_clear_object (&cache_dev_object);
