@@ -106,7 +106,6 @@ enum {
 
 const gchar *iscsi_nodes_fmt = "a(sisis)";
 const gchar *iscsi_node_fmt = "(sisis)";
-const gchar *iscsi_policy_action_id = "org.freedesktop.udisks2.iscsi.manage-iscsi";
 
 static struct libiscsi_context *
 iscsi_get_libiscsi_context (UDisksDaemon *daemon)
@@ -293,8 +292,8 @@ iscsi_login (UDisksDaemon  *daemon,
              gchar        **errorstr)
 {
   struct libiscsi_context *ctx;
-  struct libiscsi_auth_info auth_info;
-  struct libiscsi_node node;
+  struct libiscsi_auth_info auth_info = {0,};
+  struct libiscsi_node node = {0,};
   GVariant *params_without_chap;
   const gchar *username = NULL;
   const gchar *password = NULL;
@@ -354,7 +353,7 @@ iscsi_logout (UDisksDaemon  *daemon,
               gchar        **errorstr)
 {
   struct libiscsi_context *ctx;
-  struct libiscsi_node node;
+  struct libiscsi_node node = {0,};
   gint err;
 
   g_return_val_if_fail (UDISKS_IS_DAEMON (daemon), 1);
@@ -392,7 +391,7 @@ iscsi_discover_send_targets (UDisksDaemon   *daemon,
                              gchar         **errorstr)
 {
   struct libiscsi_context *ctx;
-  struct libiscsi_auth_info auth_info;
+  struct libiscsi_auth_info auth_info = {0,};
   struct libiscsi_node *found_nodes;
   const gchar *username = NULL;
   const gchar *password = NULL;
