@@ -356,7 +356,7 @@ udisks_linux_iscsi_session_object_update_iface (UDisksLinuxISCSISessionObject *s
   UDisksISCSISession *iface;
   UDisksISCSIState *state;
   struct libiscsi_context *ctx;
-  struct libiscsi_session_info session_info;
+  struct libiscsi_session_info session_info = {0,};
 
   g_return_if_fail (UDISKS_IS_LINUX_ISCSI_SESSION_OBJECT (session_object));
 
@@ -409,6 +409,7 @@ udisks_linux_iscsi_session_object_process_uevent (UDisksModuleObject *module_obj
   /* Did we get uevent for this session? */
   if (session_id && g_strcmp0 (session_id, session_object->session_id) == 0)
     {
+      g_free (session_id);
       if (g_strcmp0 (action, "remove") == 0)
         {
           /* Returning FALSE means that the device is removed. */
