@@ -81,8 +81,8 @@ G_DEFINE_TYPE_WITH_CODE (UDisksLinuxManagerISCSIInitiator, udisks_linux_manager_
                          G_IMPLEMENT_INTERFACE (UDISKS_TYPE_MANAGER_ISCSI_INITIATOR,
                                                 udisks_linux_manager_iscsi_initiator_iface_init));
 
-const gchar *initiator_filename = "/etc/iscsi/initiatorname.iscsi";
-const gchar *initiator_name_prefix = "InitiatorName=";
+#define INITIATOR_FILENAME "/etc/iscsi/initiatorname.iscsi"
+#define INITIATOR_NAME_KEY "InitiatorName"
 
 /* ---------------------------------------------------------------------------------------------------- */
 
@@ -282,73 +282,68 @@ handle_get_firmware_initiator_name (UDisksManagerISCSIInitiator *object,
   return TRUE;
 }
 
+
+#define FAKE_GROUP_NAME "general"       /* NOTE: shows up in some error messages */
+
+static gchar *
+_get_initiator_name (GError **error)
+{
+  gchar *initiator_name;
+  gchar *contents = NULL;
+  gchar *contents_group;
+  GKeyFile *key_file;
+
+  if (! g_file_get_contents (INITIATOR_FILENAME, &contents, NULL, error))
+    {
+      g_prefix_error (error, "Error reading iSCSI initiator name from '%s': ", INITIATOR_FILENAME);
+      return NULL;
+    }
+
+  /* key-value pairs have to belong to some group, let's create a fake one just to satisfy the parser needs */
+  contents_group = g_strconcat ("[" FAKE_GROUP_NAME "]\n", contents, NULL);
+  g_free (contents);
+
+  key_file = g_key_file_new ();
+  if (! g_key_file_load_from_data (key_file, contents_group, -1, G_KEY_FILE_NONE, error))
+    {
+      g_prefix_error (error, "Error reading iSCSI initiator name from '%s': ", INITIATOR_FILENAME);
+      g_key_file_free (key_file);
+      g_free (contents_group);
+      return NULL;
+    }
+
+  initiator_name = g_key_file_get_string (key_file, FAKE_GROUP_NAME, INITIATOR_NAME_KEY, error);
+  if (initiator_name == NULL)
+    g_prefix_error (error, "Error reading iSCSI initiator name from '%s': ", INITIATOR_FILENAME);
+
+  g_key_file_free (key_file);
+  g_free (contents_group);
+
+  /* trim the whitespace at the end of the string */
+  if (initiator_name != NULL)
+    initiator_name = g_strchomp (initiator_name);
+
+  return initiator_name;
+}
+
 static gboolean
 handle_get_initiator_name (UDisksManagerISCSIInitiator *object,
                            GDBusMethodInvocation       *invocation)
 {
   UDisksLinuxManagerISCSIInitiator *manager = UDISKS_LINUX_MANAGER_ISCSI_INITIATOR (object);
-
-  int initiator_name_fd = -1;
-  int nbytes;
-  gchar buf[64]; /* Do we need more? */
-  gchar *initiator_name;
-  size_t len;
-  GString *content = NULL;
+  gchar *initiator_name = NULL;
+  GError *error = NULL;
 
   /* Enter a critical section. */
   g_mutex_lock (&manager->initiator_config_mutex);
 
-  initiator_name_fd = open (initiator_filename, O_RDONLY);
-  if (initiator_name_fd == -1)
+  initiator_name = _get_initiator_name(&error);
+  if (!initiator_name)
     {
-      g_dbus_method_invocation_return_error (invocation,
-                                             UDISKS_ERROR,
-                                             UDISKS_ERROR_FAILED,
-                                             N_("Error opening %s: %s"),
-                                             initiator_filename,
-                                             strerror (errno));
-
+      g_dbus_method_invocation_take_error (invocation, error);
       goto out;
     }
 
-  /* Read the initiator name. */
-  content = g_string_new (NULL);
-  while ((nbytes = read (initiator_name_fd, buf, sizeof (buf) - 1)) > 0)
-    {
-      buf[nbytes] = '\0';
-      content = g_string_append (content, buf);
-    }
-
-  if (nbytes < 0)
-    {
-      g_dbus_method_invocation_return_error (invocation,
-                                             UDISKS_ERROR,
-                                             UDISKS_ERROR_FAILED,
-                                             N_("Error reading %s: %s"),
-                                             initiator_filename,
-                                             strerror (errno));
-
-      goto out;
-    }
-
-  /* We don't want to create a scanner for this iscsi initiator name grammar.
-   * So for simplicity, we just search for "InitiatorName=" prefix and if
-   * found, we just shift the name pointer.  The string may contain whitespace
-   * at the end; it's removed as well. */
-  len = strlen (initiator_name_prefix);
-  initiator_name = content->str;
-  if (strncmp (content->str, initiator_name_prefix, len) == 0)
-    {
-      /* Shift the pointer by prefix length further. */
-      initiator_name = &content->str[len];
-    }
-  /* Trim the whitespace at the end of the string. */
-  while (g_ascii_isspace (content->str[content->len - 1]))
-    {
-      content->str[content->len - 1] = '\0';
-      content->len--;
-    }
-
   /* Return the initiator name */
   udisks_manager_iscsi_initiator_complete_get_initiator_name (object,
                                                               invocation,
@@ -359,9 +354,41 @@ out:
   g_mutex_unlock (&manager->initiator_config_mutex);
 
   /* Release the resources */
-  g_string_free (content, TRUE);
-  if (initiator_name_fd != -1)
-    close (initiator_name_fd);
+  g_free (initiator_name);
+
+  /* Indicate that we handled the method invocation */
+  return TRUE;
+}
+
+static gboolean
+handle_get_initiator_name_raw (UDisksManagerISCSIInitiator *object,
+                               GDBusMethodInvocation       *invocation)
+{
+  UDisksLinuxManagerISCSIInitiator *manager = UDISKS_LINUX_MANAGER_ISCSI_INITIATOR (object);
+  gchar *initiator_name = NULL;
+  GError *error = NULL;
+
+  /* Enter a critical section. */
+  g_mutex_lock (&manager->initiator_config_mutex);
+
+  initiator_name = _get_initiator_name(&error);
+  if (!initiator_name)
+    {
+      g_dbus_method_invocation_take_error (invocation, error);
+      goto out;
+    }
+
+  /* Return the initiator name */
+  udisks_manager_iscsi_initiator_complete_get_initiator_name_raw (object,
+                                                                  invocation,
+                                                                  initiator_name);
+
+out:
+  /* Leave the critical section. */
+  g_mutex_unlock (&manager->initiator_config_mutex);
+
+  /* Release the resources */
+  g_free (initiator_name);
 
   /* Indicate that we handled the method invocation */
   return TRUE;
@@ -374,82 +401,87 @@ handle_set_initiator_name (UDisksManagerISCSIInitiator *object,
                            GVariant                    *arg_options)
 {
   UDisksLinuxManagerISCSIInitiator *manager = UDISKS_LINUX_MANAGER_ISCSI_INITIATOR (object);
-  int initiator_name_fd = -1;
-  GString *content = NULL;
+  gchar *contents = NULL;
+  gchar *contents_group;
+  gchar *initiator_name;
+  GKeyFile *key_file;
+  GError *error = NULL;
 
   /* Policy check. */
-  UDISKS_DAEMON_CHECK_AUTHORIZATION (manager->daemon,
-                                       NULL,
-                                       iscsi_policy_action_id,
-                                       arg_options,
-                                       N_("Authentication is required change iSCSI initiator name"),
-                                       invocation);
+  if (! udisks_daemon_util_check_authorization_sync (manager->daemon,
+                                                     NULL,
+                                                     ISCSI_MODULE_POLICY_ACTION_ID,
+                                                     arg_options,
+                                                     N_("Authentication is required change iSCSI initiator name"),
+                                                     invocation))
+    return TRUE;
 
   if (!arg_name || strlen (arg_name) == 0)
     {
-      g_dbus_method_invocation_return_error (invocation,
-                                             UDISKS_ERROR,
-                                             UDISKS_ERROR_FAILED,
-                                             N_("Empty initiator name"));
+      g_dbus_method_invocation_return_error_literal (invocation,
+                                                     UDISKS_ERROR,
+                                                     UDISKS_ERROR_FAILED,
+                                                     N_("Empty initiator name"));
       return TRUE;
     }
 
-  /* Enter a critical section. */
+  /* enter critical section */
   g_mutex_lock (&manager->initiator_config_mutex);
 
-  initiator_name_fd = open (initiator_filename,
-                            O_WRONLY |
-                            O_TRUNC  |
-                            S_IRUSR  |
-                            S_IWUSR  |
-                            S_IRGRP  |
-                            S_IROTH);
+  /* first try to read existing file */
+  g_file_get_contents (INITIATOR_FILENAME, &contents, NULL, NULL /* ignore errors */);
 
-  if (initiator_name_fd == -1)
+  /* key-value pairs have to belong to some group, let's create a fake one just to satisfy the parser needs */
+  contents_group = g_strconcat ("[" FAKE_GROUP_NAME "]\n", contents, NULL);
+  g_free (contents);
+
+  key_file = g_key_file_new ();
+  if (! g_key_file_load_from_data (key_file, contents_group, -1,
+                                   G_KEY_FILE_KEEP_COMMENTS | G_KEY_FILE_KEEP_TRANSLATIONS,
+                                   NULL /* ignore errors */))
     {
-      g_dbus_method_invocation_return_error (invocation,
-                                             UDISKS_ERROR,
-                                             UDISKS_ERROR_FAILED,
-                                             N_("Error opening %s: %s"),
-                                             initiator_filename,
-                                             strerror (errno));
-
-      goto mutex_out;
+      /* ignoring errors, leaving the keyfile instance empty (e.g. empty or non-existing config file) */
     }
+  g_free (contents_group);
 
-  /* Make a new initiator name */
-  content = g_string_new (initiator_name_prefix);
-  g_string_append_printf (content, "%s\n", arg_name);
+  /* ensure trailing space in the initiator name */
+  if (arg_name[strlen (arg_name) - 1] != ' ')
+    initiator_name = g_strconcat (arg_name, " ", NULL);
+  else
+    initiator_name = g_strdup (arg_name);
+  g_key_file_set_string (key_file, FAKE_GROUP_NAME, INITIATOR_NAME_KEY, initiator_name);
+  g_free (initiator_name);
 
-  /* Write the new initiator name */
-  if (write (initiator_name_fd, content->str, content->len) != (ssize_t) content->len)
+  contents_group = g_key_file_to_data (key_file, NULL, NULL);
+  /* strip the fake group name */
+  contents = contents_group ? g_strrstr (contents_group, "[" FAKE_GROUP_NAME "]") : NULL;
+  if (contents != NULL)
+    contents += strlen ("[" FAKE_GROUP_NAME "]\n");
+
+  if (contents == NULL)
     {
-      g_dbus_method_invocation_return_error (invocation,
-                                             UDISKS_ERROR,
-                                             UDISKS_ERROR_FAILED,
-                                             N_("Error writing to %s: %s"),
-                                             initiator_filename,
-                                             strerror (errno));
-
-      goto mutex_out;
+      g_dbus_method_invocation_return_error_literal (invocation,
+                                                     UDISKS_ERROR,
+                                                     UDISKS_ERROR_FAILED,
+                                                     N_("Error parsing the iSCSI initiator name"));
     }
+  else if (! g_file_set_contents (INITIATOR_FILENAME, contents, -1, &error))
+    {
+      g_prefix_error (&error, N_("Error writing to %s while setting iSCSI initiator name: "), INITIATOR_FILENAME);
+      g_dbus_method_invocation_take_error (invocation, error);
+    }
+  else
+    {
+      /* finish with no error */
+      udisks_manager_iscsi_initiator_complete_set_initiator_name (object, invocation);
+    }
+  g_free (contents_group);
+  g_key_file_free (key_file);
 
-  /* Finish with no error */
-  udisks_manager_iscsi_initiator_complete_set_initiator_name (object,
-                                                              invocation);
-
-mutex_out:
-  /* Leave the critical section. */
+  /* leave critical section */
   g_mutex_unlock (&manager->initiator_config_mutex);
 
-out:
-  /* Release the resources */
-  if (content)
-    g_string_free (content, TRUE);
-  if (initiator_name_fd != -1)
-    close (initiator_name_fd);
-
-  /* Indicate that we handled the method invocation */
+  /* indicate that we handled the method invocation */
   return TRUE;
 }
 
@@ -463,7 +495,6 @@ out:
  * Performs firmware discovery (ppc or ibft).
  */
 static gint
-
 discover_firmware (UDisksManagerISCSIInitiator  *object,
                    GVariant                    **nodes,
                    gint                         *nodes_cnt,
@@ -516,7 +547,7 @@ handle_discover_send_targets (UDisksManagerISCSIInitiator *object,
   /* Policy check. */
   UDISKS_DAEMON_CHECK_AUTHORIZATION (manager->daemon,
                                      NULL,
-                                     iscsi_policy_action_id,
+                                     ISCSI_MODULE_POLICY_ACTION_ID,
                                      arg_options,
                                      N_("Authentication is required to discover targets"),
                                      invocation);
@@ -574,7 +605,7 @@ handle_discover_firmware (UDisksManagerISCSIInitiator *object,
   /* Policy check. */
   UDISKS_DAEMON_CHECK_AUTHORIZATION (manager->daemon,
                                      NULL,
-                                     iscsi_policy_action_id,
+                                     ISCSI_MODULE_POLICY_ACTION_ID,
                                      arg_options,
                                      N_("Authentication is required to discover firmware targets"),
                                      invocation);
@@ -623,11 +654,13 @@ handle_login (UDisksManagerISCSIInitiator *object,
   gint err = 0;
   gchar *errorstr = NULL;
   GError *error = NULL;
+  UDisksObject *iscsi_object = NULL;
+  UDisksObject *iscsi_session_object = NULL;
 
   /* Policy check. */
   UDISKS_DAEMON_CHECK_AUTHORIZATION (manager->daemon,
                                      NULL,
-                                     iscsi_policy_action_id,
+                                     ISCSI_MODULE_POLICY_ACTION_ID,
                                      arg_options,
                                      N_("Authentication is required to perform iSCSI login"),
                                      invocation);
@@ -660,12 +693,13 @@ handle_login (UDisksManagerISCSIInitiator *object,
     }
 
   /* sit and wait until the device appears on dbus */
-  if (udisks_daemon_wait_for_object_sync (manager->daemon,
-                                          wait_for_iscsi_object,
-                                          g_strdup (arg_name),
-                                          g_free,
-                                          15, /* timeout_seconds */
-                                          &error) == NULL)
+  iscsi_object = udisks_daemon_wait_for_object_sync (manager->daemon,
+                                                     wait_for_iscsi_object,
+                                                     g_strdup (arg_name),
+                                                     g_free,
+                                                     15, /* timeout_seconds */
+                                                     &error);
+   if (iscsi_object == NULL)
     {
       g_prefix_error (&error, "Error waiting for iSCSI device to appear: ");
       g_dbus_method_invocation_take_error (invocation, error);
@@ -674,12 +708,13 @@ handle_login (UDisksManagerISCSIInitiator *object,
 
   if (udisks_manager_iscsi_initiator_get_sessions_supported (UDISKS_MANAGER_ISCSI_INITIATOR (manager)))
     {
-      if (udisks_daemon_wait_for_object_sync (manager->daemon,
-                                              wait_for_iscsi_session_object,
-                                              g_strdup (arg_name),
-                                              g_free,
-                                              15, /* timeout_seconds */
-                                              &error) == NULL)
+      iscsi_session_object = udisks_daemon_wait_for_object_sync (manager->daemon,
+                                                                 wait_for_iscsi_session_object,
+                                                                 g_strdup (arg_name),
+                                                                 g_free,
+                                                                 15, /* timeout_seconds */
+                                                                 &error);
+      if (iscsi_session_object == NULL)
         {
           g_prefix_error (&error, "Error waiting for iSCSI session object to appear: ");
           g_dbus_method_invocation_take_error (invocation, error);
@@ -692,6 +727,8 @@ handle_login (UDisksManagerISCSIInitiator *object,
                                                  invocation);
 
 out:
+  g_clear_object (&iscsi_object);
+  g_clear_object (&iscsi_session_object);
   g_free ((gpointer) errorstr);
 
   /* Indicate that we handled the method invocation. */
@@ -717,7 +754,7 @@ handle_logout(UDisksManagerISCSIInitiator *object,
   /* Policy check. */
   UDISKS_DAEMON_CHECK_AUTHORIZATION (manager->daemon,
                                      NULL,
-                                     iscsi_policy_action_id,
+                                     ISCSI_MODULE_POLICY_ACTION_ID,
                                      arg_options,
                                      N_("Authentication is required to perform iSCSI logout"),
                                      invocation);
@@ -795,6 +832,7 @@ udisks_linux_manager_iscsi_initiator_iface_init (UDisksManagerISCSIInitiatorIfac
 {
   iface->handle_get_firmware_initiator_name = handle_get_firmware_initiator_name;
   iface->handle_get_initiator_name = handle_get_initiator_name;
+  iface->handle_get_initiator_name_raw = handle_get_initiator_name_raw;
   iface->handle_set_initiator_name = handle_set_initiator_name;
   iface->handle_discover_send_targets = handle_discover_send_targets;
   iface->handle_discover_firmware = handle_discover_firmware;
