@@ -175,6 +175,7 @@ handle_poll (UDisksVolumeGroup *_group,
 
 /* ---------------------------------------------------------------------------------------------------- */
 
+/*  transfer-full  */
 GList *
 udisks_linux_volume_group_get_logical_volumes (UDisksVolumeGroup *group,
                                                UDisksDaemon      *daemon)
@@ -249,7 +250,6 @@ handle_delete (UDisksVolumeGroup     *_group,
   UDisksLinuxVolumeGroupObject *object = NULL;
   UDisksDaemon *daemon;
   uid_t caller_uid;
-  gid_t caller_gid;
   gboolean teardown_flag = FALSE;
   GList *objects_to_wipe = NULL;
   GList *l;
@@ -286,8 +286,6 @@ handle_delete (UDisksVolumeGroup     *_group,
                                                invocation,
                                                NULL /* GCancellable */,
                                                &caller_uid,
-                                               &caller_gid,
-                                               NULL,
                                                &error))
     {
       g_dbus_method_invocation_return_gerror (invocation, error);
@@ -331,6 +329,7 @@ handle_delete (UDisksVolumeGroup     *_group,
                                              UDISKS_ERROR_FAILED,
                                              "Error deleting volume group: %s",
                                              error->message);
+      g_clear_error (&error);
       goto out;
     }
 
@@ -356,8 +355,14 @@ wait_for_volume_group_object (UDisksDaemon *daemon,
                               gpointer      user_data)
 {
   const gchar *name = user_data;
+  UDisksLinuxVolumeGroupObject *object;
 
-  return UDISKS_OBJECT (udisks_daemon_util_lvm2_find_volume_group_object (daemon, name));
+  object = udisks_daemon_util_lvm2_find_volume_group_object (daemon, name);
+
+  if (object == NULL)
+    return NULL;
+
+  return g_object_ref (UDISKS_OBJECT (object));
 }
 
 static gboolean
@@ -371,7 +376,6 @@ handle_rename (UDisksVolumeGroup     *_group,
   UDisksLinuxVolumeGroupObject *object = NULL;
   UDisksDaemon *daemon;
   uid_t caller_uid;
-  gid_t caller_gid;
   UDisksObject *group_object = NULL;
   VGJobData data;
 
@@ -388,8 +392,6 @@ handle_rename (UDisksVolumeGroup     *_group,
                                                invocation,
                                                NULL /* GCancellable */,
                                                &caller_uid,
-                                               &caller_gid,
-                                               NULL,
                                                &error))
     {
       g_dbus_method_invocation_return_gerror (invocation, error);
@@ -423,6 +425,7 @@ handle_rename (UDisksVolumeGroup     *_group,
                                              UDISKS_ERROR_FAILED,
                                              "Error renaming volume group: %s",
                                              error->message);
+      g_clear_error (&error);
       goto out;
     }
 
@@ -435,7 +438,7 @@ handle_rename (UDisksVolumeGroup     *_group,
   if (group_object == NULL)
     {
       g_prefix_error (&error,
-                      "Error waiting for volume group object for %s",
+                      "Error waiting for volume group object for '%s': ",
                       new_name);
       g_dbus_method_invocation_take_error (invocation, error);
       goto out;
@@ -444,6 +447,7 @@ handle_rename (UDisksVolumeGroup     *_group,
   udisks_volume_group_complete_rename (_group,
                                        invocation,
                                        g_dbus_object_get_object_path (G_DBUS_OBJECT (group_object)));
+  g_clear_object (&group_object);
 
  out:
   g_clear_object (&object);
@@ -462,7 +466,6 @@ handle_add_device (UDisksVolumeGroup     *_group,
   UDisksDaemon *daemon;
   UDisksLinuxVolumeGroupObject *object;
   uid_t caller_uid;
-  gid_t caller_gid;
   GError *error = NULL;
   UDisksObject *new_member_device_object = NULL;
   UDisksBlock *new_member_device = NULL;
@@ -483,8 +486,6 @@ handle_add_device (UDisksVolumeGroup     *_group,
                                                invocation,
                                                NULL /* GCancellable */,
                                                &caller_uid,
-                                               &caller_gid,
-                                               NULL,
                                                &error))
     {
       g_dbus_method_invocation_return_gerror (invocation, error);
@@ -549,6 +550,7 @@ handle_add_device (UDisksVolumeGroup     *_group,
                                                  "Error creating LVM metadata on %s: %s",
                                                  pv_data.path,
                                                  error->message);
+          g_clear_error (&error);
           goto out;
         }
     }
@@ -573,6 +575,7 @@ handle_add_device (UDisksVolumeGroup     *_group,
                                              "Error adding %s to volume group: %s",
                                              data.pv_path,
                                              error->message);
+      g_clear_error (&error);
       goto out;
     }
 
@@ -598,7 +601,6 @@ handle_remove_common (UDisksVolumeGroup     *_group,
   UDisksDaemon *daemon;
   UDisksLinuxVolumeGroupObject *object;
   uid_t caller_uid;
-  gid_t caller_gid;
   GError *error = NULL;
   UDisksObject *member_device_object = NULL;
   UDisksBlock *member_device = NULL;
@@ -638,8 +640,6 @@ handle_remove_common (UDisksVolumeGroup     *_group,
                                                invocation,
                                                NULL /* GCancellable */,
                                                &caller_uid,
-                                               &caller_gid,
-                                               NULL,
                                                &error))
     {
       g_dbus_method_invocation_return_gerror (invocation, error);
@@ -692,6 +692,7 @@ handle_remove_common (UDisksVolumeGroup     *_group,
                                              (is_remove) ? "Error remove %s from volume group: %s" : "Error emptying %s: %s",
                                              data.pv_path,
                                              error->message);
+      g_clear_error (&error);
       goto out;
     }
 
@@ -714,6 +715,7 @@ handle_remove_common (UDisksVolumeGroup     *_group,
                                                  data.pv_path,
                                                  udisks_linux_volume_group_object_get_name (object),
                                                  error->message);
+          g_clear_error (&error);
           goto out;
         }
     }
@@ -763,8 +765,13 @@ wait_for_logical_volume_object (UDisksDaemon *daemon,
                                 gpointer      user_data)
 {
   struct WaitData *data = user_data;
-  return UDISKS_OBJECT (udisks_linux_volume_group_object_find_logical_volume_object (data->group_object,
-                                                                                     data->name));
+  UDisksLinuxLogicalVolumeObject *object;
+
+  object = udisks_linux_volume_group_object_find_logical_volume_object (data->group_object, data->name);
+  if (object == NULL)
+    return NULL;
+
+  return g_object_ref (UDISKS_OBJECT (object));
 }
 
 static const gchar *
@@ -775,6 +782,7 @@ wait_for_logical_volume_path (UDisksLinuxVolumeGroupObject  *group_object,
   struct WaitData data;
   UDisksDaemon *daemon;
   UDisksObject *volume_object;
+  const gchar *object_path;
 
   data.group_object = group_object;
   data.name = name;
@@ -788,7 +796,10 @@ wait_for_logical_volume_path (UDisksLinuxVolumeGroupObject  *group_object,
   if (volume_object == NULL)
     return NULL;
 
-  return g_dbus_object_get_object_path (G_DBUS_OBJECT (volume_object));
+  object_path = g_dbus_object_get_object_path (G_DBUS_OBJECT (volume_object));
+  g_object_unref (volume_object);
+
+  return object_path;
 }
 
 /* ---------------------------------------------------------------------------------------------------- */
@@ -812,7 +823,6 @@ handle_create_volume (UDisksVolumeGroup              *_group,
   UDisksLinuxVolumeGroupObject *object = NULL;
   UDisksDaemon *daemon;
   uid_t caller_uid;
-  gid_t caller_gid;
   const gchar *lv_objpath;
   LVJobData data;
   UDisksLinuxLogicalVolumeObject *pool_object = NULL;
@@ -852,8 +862,6 @@ handle_create_volume (UDisksVolumeGroup              *_group,
                                                invocation,
                                                NULL /* GCancellable */,
                                                &caller_uid,
-                                               &caller_gid,
-                                               NULL,
                                                &error))
     {
       g_dbus_method_invocation_return_gerror (invocation, error);
@@ -903,6 +911,7 @@ handle_create_volume (UDisksVolumeGroup              *_group,
                                              UDISKS_ERROR_FAILED,
                                              "Error creating volume: %s",
                                              error->message);
+      g_clear_error (&error);
       goto out;
     }
 
@@ -910,7 +919,7 @@ handle_create_volume (UDisksVolumeGroup              *_group,
   if (lv_objpath == NULL)
     {
       g_prefix_error (&error,
-                      "Error waiting for logical volume object for %s",
+                      "Error waiting for logical volume object for '%s': ",
                       arg_name);
       g_dbus_method_invocation_take_error (invocation, error);
       goto out;
