@@ -568,6 +568,7 @@ update_vg (GObject      *source_obj,
           udisks_warning ("Failed to update LVM volume group %s: %s",
                           udisks_linux_volume_group_object_get_name (object),
                           error->message);
+          g_clear_error (&error);
         }
       else
         {
@@ -616,7 +617,7 @@ update_vg (GObject      *source_obj,
           udisks_linux_logical_volume_object_update (volume, lv_info, meta_lv_info, &needs_polling);
           udisks_linux_logical_volume_object_update_etctabs (volume);
           g_dbus_object_manager_server_export_uniquely (manager, G_DBUS_OBJECT_SKELETON (volume));
-          g_hash_table_insert (object->logical_volumes, g_strdup (lv_name), g_object_ref (volume));
+          g_hash_table_insert (object->logical_volumes, g_strdup (lv_name), volume);
         }
       else
         udisks_linux_logical_volume_object_update (volume, lv_info, meta_lv_info, &needs_polling);
@@ -635,8 +636,6 @@ update_vg (GObject      *source_obj,
           g_dbus_object_manager_server_unexport (manager,
                                                  g_dbus_object_get_object_path (G_DBUS_OBJECT (volume)));
           g_hash_table_iter_remove (&volume_iter);
-
-          g_object_unref (G_OBJECT (volume));
         }
     }
 
@@ -715,9 +714,12 @@ poll_vg_update (GObject      *source_obj,
   if (!lvs)
     {
       if (error)
-        udisks_warning ("Failed to poll LVM volume group %s: %s",
-                        udisks_linux_volume_group_object_get_name (object),
-                        error->message);
+        {
+          udisks_warning ("Failed to poll LVM volume group %s: %s",
+                          udisks_linux_volume_group_object_get_name (object),
+                          error->message);
+          g_clear_error (&error);
+        }
       else
         /* this should never happen */
         udisks_warning ("Failed to poll LVM volume group %s: no error reported",
