@@ -26,6 +26,8 @@
 
 #include "udiskszramutil.h"
 
+#define BUFLEN 256
+
 const gchar *zram_policy_action_id = "org.freedesktop.udisks2.zram.manage-zram";
 
 gboolean
@@ -36,9 +38,10 @@ set_conf_property (char *filename,
 {
   FILE *f = NULL;
   FILE  *tmp = NULL;
-  char buff[256];
+  char buff[BUFLEN];
   gchar* tmpfname;
   gboolean newprop = TRUE;
+  gint fd;
 
   f = fopen (filename, "r+");
   if (f == NULL)
@@ -48,7 +51,15 @@ set_conf_property (char *filename,
     }
 
   tmpfname = g_strdup_printf ("%sXXXXXX", filename);
-  mkstemp (tmpfname);
+  fd = g_mkstemp (tmpfname);
+  if (fd == -1)
+    {
+      g_set_error (error, G_IO_ERROR, g_io_error_from_errno (errno), "%m");
+      fclose (f);
+      g_free (tmpfname);
+      return FALSE;
+    }
+
   if (chmod (tmpfname, S_IRUSR | S_IWUSR | S_IRGRP | S_IROTH) != 0)
     {
       g_set_error (error, G_IO_ERROR, g_io_error_from_errno (errno),"%m");
@@ -57,20 +68,21 @@ set_conf_property (char *filename,
       return FALSE;
     }
 
-  tmp = fopen (tmpfname, "w");
+  tmp = fdopen (fd, "w");
   if (tmp == NULL)
     {
       g_set_error (error, G_IO_ERROR, g_io_error_from_errno (errno),"%m");
       fclose (f);
+      close (fd);
       g_free (tmpfname);
       return FALSE;
     }
 
-  while (fgets (buff, 255, f))
+  while (fgets (buff, BUFLEN, f))
     {
       if (! strncmp(key, buff, strlen(key)))
         {
-          strncpy (buff+strlen (key)+1, value, 255-strlen (key));
+          strncpy (buff+strlen (key)+1, value, BUFLEN-strlen (key)-1);
           buff[strlen (buff)] = '\n';
           newprop = FALSE;
         }
