@@ -96,6 +96,15 @@ udisks_daemon_util_subst_str (const gchar *str,
     return result;
 }
 
+/**
+ * udisks_daemon_util_subst_str_and_escape:
+ * @str: A command string with variables to replace.
+ * @from: A variable name that should be expanded.
+ * @to: The value for that variable. Will be escaped for shells before
+ * substitution.
+ *
+ * Returns: The command string with the substituted variable.
+ */
 gchar *
 udisks_daemon_util_subst_str_and_escape (const gchar *str,
                       const gchar *from,
@@ -103,7 +112,7 @@ udisks_daemon_util_subst_str_and_escape (const gchar *str,
 {
   gchar *quoted_and_escaped;
   gchar *ret;
-  quoted_and_escaped = udisks_daemon_util_escape_and_quote (to);
+  quoted_and_escaped = g_shell_quote (to);
   ret = udisks_daemon_util_subst_str (str, from, quoted_and_escaped);
   g_free (quoted_and_escaped);
   return ret;
@@ -146,7 +155,13 @@ udisks_variant_lookup_binary (GVariant     *dict,
 {
   GVariant* item = g_variant_lookup_value (dict, name, NULL);
   if (item)
-    return udisks_variant_get_binary (item, out_text);
+    {
+      gboolean ret = udisks_variant_get_binary (item, out_text);
+      g_variant_unref (item);
+
+      return ret;
+    }
+
   return FALSE;
 }
 
@@ -1159,75 +1174,6 @@ udisks_daemon_util_dup_object (gpointer   interface_,
   return ret;
 }
 
-static void
-escaper (GString *s, const gchar *str)
-{
-  const gchar *p;
-  for (p = str; *p != '\0'; p++)
-    {
-      gint c = *p;
-      switch (c)
-        {
-        case '"':
-          g_string_append (s, "\\\"");
-          break;
-
-        case '\\':
-          g_string_append (s, "\\\\");
-          break;
-
-        default:
-          g_string_append_c (s, c);
-          break;
-        }
-    }
-}
-
-/**
- * udisks_daemon_util_escape_and_quote:
- * @str: The string to escape.
- *
- * Like udisks_daemon_util_escape() but also wraps the result in
- * double-quotes.
- *
- * Returns: The double-quoted and escaped string. Free with g_free().
- */
-gchar *
-udisks_daemon_util_escape_and_quote (const gchar *str)
-{
-  GString *s;
-
-  g_return_val_if_fail (str != NULL, NULL);
-
-  s = g_string_new ("\"");
-  escaper (s, str);
-  g_string_append_c (s, '"');
-
-  return g_string_free (s, FALSE);
-}
-
-/**
- * udisks_daemon_util_escape:
- * @str: The string to escape.
- *
- * Escapes double-quotes (&quot;) and back-slashes (\) in a string
- * using back-slash (\).
- *
- * Returns: The escaped string. Free with g_free().
- */
-gchar *
-udisks_daemon_util_escape (const gchar *str)
-{
-  GString *s;
-
-  g_return_val_if_fail (str != NULL, NULL);
-
-  s = g_string_new (NULL);
-  escaper (s, str);
-
-  return g_string_free (s, FALSE);
-}
-
 /**
  * udisks_daemon_util_on_user_seat:
  * @daemon: A #UDisksDaemon.
