@@ -98,8 +98,7 @@ udisks_linux_drive_object_finalize (GObject *_object)
   UDisksLinuxDriveObject *object = UDISKS_LINUX_DRIVE_OBJECT (_object);
 
   /* note: we don't hold a ref to drive_object->daemon or drive_object->mount_monitor */
-  g_list_foreach (object->devices, (GFunc) g_object_unref, NULL);
-  g_list_free (object->devices);
+  g_list_free_full (object->devices, g_object_unref);
 
   if (object->iface_drive != NULL)
     g_object_unref (object->iface_drive);
@@ -399,8 +398,7 @@ udisks_linux_drive_object_get_devices (UDisksLinuxDriveObject *object)
 {
   GList *ret;
   g_return_val_if_fail (UDISKS_IS_LINUX_DRIVE_OBJECT (object), NULL);
-  ret = g_list_copy (object->devices);
-  g_list_foreach (ret, (GFunc) g_object_ref, NULL);
+  ret = g_list_copy_deep (object->devices, (GCopyFunc) udisks_g_object_ref_copy, NULL);
   return ret;
 }
 
@@ -484,14 +482,13 @@ udisks_linux_drive_object_get_block (UDisksLinuxDriveObject *object,
       if (g_strcmp0 (udisks_block_get_drive (block),
                      g_dbus_object_get_object_path (G_DBUS_OBJECT (object))) == 0)
         {
-          ret = g_object_ref (iter_object);
+          ret = UDISKS_LINUX_BLOCK_OBJECT (g_object_ref (iter_object));
           goto out;
         }
     }
 
  out:
-  g_list_foreach (objects, (GFunc) g_object_unref, NULL);
-  g_list_free (objects);
+  g_list_free_full (objects, g_object_unref);
   return ret;
 }
 
@@ -1209,8 +1206,7 @@ udisks_linux_drive_object_get_siblings (UDisksLinuxDriveObject *object)
 
  out:
   ret = g_list_reverse (ret);
-  g_list_foreach (objects, (GFunc) g_object_unref, NULL);
-  g_list_free (objects);
+  g_list_free_full (objects, g_object_unref);
   g_free (sibling_id);
   return ret;
 }
