@@ -303,8 +303,8 @@ udisks_client_new (GCancellable        *cancellable,
  *
  * Finishes an operation started with udisks_client_new().
  *
- * Returns: A #UDisksClient or %NULL if @error is set. Free with
- * g_object_unref() when done with it.
+ * Returns: (transfer full): A #UDisksClient or %NULL if @error is set. Free
+ * with g_object_unref() when done with it.
  */
 UDisksClient *
 udisks_client_new_finish (GAsyncResult        *res,
@@ -328,8 +328,8 @@ udisks_client_new_finish (GAsyncResult        *res,
  *
  * Synchronously gets a #UDisksClient for the local system.
  *
- * Returns: A #UDisksClient or %NULL if @error is set. Free with
- * g_object_unref() when done with it.
+ * Returns: (transfer full): A #UDisksClient or %NULL if @error is set. Free
+ * with g_object_unref() when done with it.
  */
 UDisksClient *
 udisks_client_new_sync (GCancellable  *cancellable,
@@ -397,11 +397,9 @@ initable_init (GInitable     *initable,
         {
           init_interface_proxy (client, G_DBUS_PROXY (ll->data));
         }
-      g_list_foreach (interfaces, (GFunc) g_object_unref, NULL);
-      g_list_free (interfaces);
+      g_list_free_full (interfaces, g_object_unref);
     }
-  g_list_foreach (objects, (GFunc) g_object_unref, NULL);
-  g_list_free (objects);
+  g_list_free_full (objects, g_object_unref);
 
   g_signal_connect (client->object_manager,
                     "object-added",
@@ -603,8 +601,7 @@ udisks_client_get_block_for_label (UDisksClient        *client,
         g_object_unref (block);
     }
 
-  g_list_foreach (object_proxies, (GFunc) g_object_unref, NULL);
-  g_list_free (object_proxies);
+  g_list_free_full (object_proxies, g_object_unref);
   ret = g_list_reverse (ret);
   return ret;
 }
@@ -648,8 +645,7 @@ udisks_client_get_block_for_uuid (UDisksClient        *client,
         g_object_unref (block);
     }
 
-  g_list_foreach (object_proxies, (GFunc) g_object_unref, NULL);
-  g_list_free (object_proxies);
+  g_list_free_full (object_proxies, g_object_unref);
   ret = g_list_reverse (ret);
   return ret;
 }
@@ -696,8 +692,7 @@ udisks_client_get_block_for_dev (UDisksClient *client,
     }
 
  out:
-  g_list_foreach (object_proxies, (GFunc) g_object_unref, NULL);
-  g_list_free (object_proxies);
+  g_list_free_full (object_proxies, g_object_unref);
   return ret;
 }
 
@@ -734,8 +729,7 @@ get_top_level_blocks_for_drive (UDisksClient *client,
         }
       g_object_unref (block);
     }
-  g_list_foreach (object_proxies, (GFunc) g_object_unref, NULL);
-  g_list_free (object_proxies);
+  g_list_free_full (object_proxies, g_object_unref);
   return ret;
 }
 
@@ -789,8 +783,7 @@ udisks_client_get_block_for_drive (UDisksClient        *client,
     }
 
  out:
-  g_list_foreach (blocks, (GFunc) g_object_unref, NULL);
-  g_list_free (blocks);
+  g_list_free_full (blocks, g_object_unref);
   return ret;
 }
 
@@ -892,7 +885,10 @@ _udisks_client_get_block_or_blocks_for_mdraid (UDisksClient *client,
       if (skip_partitions)
         {
           if (udisks_object_peek_partition (object) != NULL)
-            continue;
+            {
+              g_object_unref (block);
+              continue;
+            }
         }
 
       if (g_strcmp0 (member_get (block), raid_objpath) == 0)
@@ -909,8 +905,7 @@ _udisks_client_get_block_or_blocks_for_mdraid (UDisksClient *client,
     }
 
  out:
-  g_list_foreach (object_proxies, (GFunc) g_object_unref, NULL);
-  g_list_free (object_proxies);
+  g_list_free_full (object_proxies, g_object_unref);
   return ret;
 }
 
@@ -1179,8 +1174,7 @@ udisks_client_get_cleartext_block (UDisksClient  *client,
     }
 
  out:
-  g_list_foreach (objects, (GFunc) g_object_unref, NULL);
-  g_list_free (objects);
+  g_list_free_full (objects, g_object_unref);
   return ret;
 }
 
@@ -1234,8 +1228,7 @@ udisks_client_get_drive_siblings  (UDisksClient  *client,
     }
   ret = g_list_reverse (ret);
  out:
-  g_list_foreach (object_proxies, (GFunc) g_object_unref, NULL);
-  g_list_free (object_proxies);
+  g_list_free_full (object_proxies, g_object_unref);
   return ret;
 }
 
@@ -1286,8 +1279,7 @@ udisks_client_get_partitions (UDisksClient         *client,
     }
   ret = g_list_reverse (ret);
  out:
-  g_list_foreach (object_proxies, (GFunc) g_object_unref, NULL);
-  g_list_free (object_proxies);
+  g_list_free_full (object_proxies, g_object_unref);
   return ret;
 }
 
@@ -1427,8 +1419,7 @@ udisks_client_get_jobs_for_object (UDisksClient  *client,
     }
   ret = g_list_reverse (ret);
 
-  g_list_foreach (object_proxies, (GFunc) g_object_unref, NULL);
-  g_list_free (object_proxies);
+  g_list_free_full (object_proxies, g_object_unref);
   return ret;
 }
 
@@ -1502,8 +1493,7 @@ on_object_added (GDBusObjectManager  *manager,
     {
       init_interface_proxy (client, G_DBUS_PROXY (l->data));
     }
-  g_list_foreach (interfaces, (GFunc) g_object_unref, NULL);
-  g_list_free (interfaces);
+  g_list_free_full (interfaces, g_object_unref);
 
   udisks_client_queue_changed (client);
 }
@@ -1685,7 +1675,7 @@ get_pow10_size (guint64 size)
  *
  * Utility function to get a human-readable string that represents @size.
  *
- * Returns: A string that should be freed with g_free().
+ * Returns: (transfer full): A string that should be freed with g_free().
  */
 gchar *
 udisks_client_get_size_for_display (UDisksClient  *client,
@@ -1746,8 +1736,8 @@ udisks_client_get_size_for_display (UDisksClient  *client,
  * Gets a human-readable string of the media described by
  * @media_compat. The returned information is localized.
  *
- * Returns: A string that should be freed with g_free() or %NULL if
- * unknown.
+ * Returns: (transfer full): A string that should be freed with g_free() or
+ * %NULL if unknown.
  */
 gchar *
 udisks_client_get_media_compat_for_display (UDisksClient       *client,
@@ -1908,6 +1898,7 @@ static const struct
   {"filesystem", "ntfs",              NULL,    NC_("fs-type", "NTFS"),                              NC_("fs-type", "NTFS")},
   {"filesystem", "hfs",               NULL,    NC_("fs-type", "HFS"),                               NC_("fs-type", "HFS")},
   {"filesystem", "hfsplus",           NULL,    NC_("fs-type", "HFS+"),                              NC_("fs-type", "HFS+")},
+  {"filesystem", "apfs",              NULL,    NC_("fs-type", "APFS"),                              NC_("fs-type", "APFS")},
   {"filesystem", "ext2",              "*",     NC_("fs-type", "Ext2 (version %s)"),                 NC_("fs-type", "Ext2")},
   {"filesystem", "ext2",              NULL,    NC_("fs-type", "Ext2"),                              NC_("fs-type", "Ext2")},
   {"filesystem", "ext3",              "*",     NC_("fs-type", "Ext3 (version %s)"),                 NC_("fs-type", "Ext3")},
@@ -1936,8 +1927,10 @@ static const struct
   {"raid",       "zfs_member",        NULL,    NC_("fs-type", "ZFS Device"),                        NC_("fs-type", "ZFS")},
   {"raid",       "isw_raid_member",   "*",     NC_("fs-type", "Intel Rapid Storage Technology enterprise RAID Member (version %s)"), NC_("fs-type", "Intel RSTe RAID Member (%s)")},
   {"raid",       "isw_raid_member",   NULL,    NC_("fs-type", "Intel Rapid Storage Technology enterprise RAID Member"),          NC_("fs-type", "Intel RSTe RAID Member")},
+  {"crypto",     "crypto_unknown",    NULL,    NC_("fs-type", "Possibly Encrypted"),                NC_("fs-type", "Encrypted?")},
   {"crypto",     "crypto_LUKS",       "*",     NC_("fs-type", "LUKS Encryption (version %s)"),      NC_("fs-type", "LUKS")},
   {"crypto",     "crypto_LUKS",       NULL,    NC_("fs-type", "LUKS Encryption"),                   NC_("fs-type", "LUKS")},
+  {"crypto",     "crypto_TCRYPT",     NULL,    NC_("fs-type", "VeraCrypt/TrueCrypt Encryption"),    NC_("fs-type", "VeraCrypt/TrueCrypt")},
   {"filesystem", "VMFS",              "*",     NC_("fs-type", "VMFS (version %s)"),                 NC_("fs-type", "VMFS (v%s)")},
   {"filesystem", "VMFS",              NULL,    NC_("fs-type", "VMFS"),                              NC_("fs-type", "VMFS")},
   {"raid",       "VMFS_volume_member", "*",    NC_("fs-type", "VMFS Volume Member (version %s)"),   NC_("fs-type", "VMFS Member (v%s)")},
@@ -1955,7 +1948,7 @@ static const struct
  *
  * Gets a human readable localized string for @usage, @type and @version.
  *
- * Returns: A string that should be freed with g_free().
+ * Returns: (transfer full): A string that should be freed with g_free().
  */
 gchar *
 udisks_client_get_id_for_display (UDisksClient *client,
@@ -2078,7 +2071,7 @@ static const struct
  *
  * Gets a human readable localized string for @partition_table_type.
  *
- * Returns: A description of @partition_table_type or %NULL.
+ * Returns: (transfer none): A description of @partition_table_type or %NULL.
  */
 const gchar *
 udisks_client_get_partition_table_type_for_display (UDisksClient  *client,
@@ -2134,7 +2127,8 @@ static const struct
  *
  * Gets a human readable localized string for @partition_table_type and @partition_table_subtype.
  *
- * Returns: A description of @partition_table_type and @partition_table_subtype or %NULL.
+ * Returns: (transfer none): A description of @partition_table_type and
+ * @partition_table_subtype or %NULL.
  */
 const gchar *
 udisks_client_get_partition_table_subtype_for_display (UDisksClient  *client,
@@ -2235,6 +2229,7 @@ static const struct
   {"gpt", "microsoft", "de94bba4-06d1-4d40-a16a-bfd50179d6ac", NC_("part-type", "Microsoft Windows Recovery Environment"), 0},
   /* Apple OS X */
   {"gpt", "apple",     "48465300-0000-11aa-aa11-00306543ecac", NC_("part-type", "Apple HFS/HFS+"), 0},
+  {"gpt", "apple",     "7c3457ef-0000-11aa-aa11-00306543ecac", NC_("part-type", "Apple APFS"), 0},
   {"gpt", "apple",     "55465300-0000-11aa-aa11-00306543ecac", NC_("part-type", "Apple UFS"), 0},
   {"gpt", "apple",     "6a898cc3-1dd2-11b2-99a6-080020736631", NC_("part-type", "Apple ZFS"), 0}, /* same as ZFS */
   {"gpt", "apple",     "52414944-0000-11aa-aa11-00306543ecac", NC_("part-type", "Apple RAID"), F_RAID},
@@ -2390,7 +2385,8 @@ udisks_client_get_partition_type_infos (UDisksClient   *client,
  *
  * Gets a human readable localized string for @partiton_table_type and @partition_type.
  *
- * Returns: A description of @partition_type or %NULL if unknown.
+ * Returns: (transfer none): A description of @partition_type or %NULL if
+ * unknown.
  */
 const gchar *
 udisks_client_get_partition_type_for_display (UDisksClient  *client,
@@ -2426,7 +2422,8 @@ udisks_client_get_partition_type_for_display (UDisksClient  *client,
  * useful in scenarios where different subtypes is using the same
  * partition type.
  *
- * Returns: A description of @partition_type or %NULL if unknown.
+ * Returns: (transfer none): A description of @partition_type or %NULL if
+ * unknown.
  *
  * Since: 2.1.1
  */
@@ -2459,30 +2456,25 @@ udisks_client_get_partition_type_and_subtype_for_display (UDisksClient  *client,
 /* ---------------------------------------------------------------------------------------------------- */
 
 /**
- * udisks_client_get_job_description:
- * @client: A #UDisksClient.
- * @job: A #UDisksJob.
+ * udisks_client_get_job_description_from_operation:
+ * @operation: A job operation name.
  *
- * Gets a human-readable and localized text string describing the
- * operation of @job.
+ * Gets a human-readable and localized text string describing a
+ * a job @operation.
  *
- * For known job types, see the documentation for the
+ * For known job operation types, see the documentation for the
  * <link linkend="gdbus-property-org-freedesktop-UDisks2-Job.Operation">Job:Operation</link>
  * D-Bus property.
  *
- * Returns: A string that should be freed with g_free().
+ * Returns: (transfer full): A string that should be freed with g_free().
  */
 gchar *
-udisks_client_get_job_description (UDisksClient   *client,
-                                   UDisksJob      *job)
+udisks_client_get_job_description_from_operation (const gchar *operation)
 {
   static gsize once = 0;
   static GHashTable *hash = NULL;
-  const gchar *operation = NULL;
   gchar *ret = NULL;
 
-  g_return_val_if_fail (UDISKS_IS_CLIENT (client), NULL);
-
   if (g_once_init_enter (&once))
     {
       hash = g_hash_table_new (g_str_hash, g_str_equal);
@@ -2491,6 +2483,7 @@ udisks_client_get_job_description (UDisksClient   *client,
       g_hash_table_insert (hash, (gpointer) "encrypted-unlock",     (gpointer) C_("job", "Unlocking Device"));
       g_hash_table_insert (hash, (gpointer) "encrypted-lock",       (gpointer) C_("job", "Locking Device"));
       g_hash_table_insert (hash, (gpointer) "encrypted-modify",     (gpointer) C_("job", "Modifying Encrypted Device"));
+      g_hash_table_insert (hash, (gpointer) "encrypted-resize",     (gpointer) C_("job", "Resizing Encrypted Device"));
       g_hash_table_insert (hash, (gpointer) "swapspace-start",      (gpointer) C_("job", "Starting Swap Device"));
       g_hash_table_insert (hash, (gpointer) "swapspace-stop",       (gpointer) C_("job", "Stopping Swap Device"));
       g_hash_table_insert (hash, (gpointer) "swapspace-modify",     (gpointer) C_("job", "Modifying Swap Device"));
@@ -2519,15 +2512,37 @@ udisks_client_get_job_description (UDisksClient   *client,
       g_once_init_leave (&once, (gsize) 1);
     }
 
-  operation = udisks_job_get_operation (job);
   if (operation != NULL)
     ret = g_strdup (g_hash_table_lookup (hash, operation));
   if (ret == NULL)
-    ret = g_strdup_printf (C_("unknown-job", "Unknown (%s)"), udisks_job_get_operation (job));
+    ret = g_strdup_printf (C_("unknown-job", "Unknown (%s)"), operation == NULL ? "(null)" : operation);
 
   return ret;
 }
 
+/**
+ * udisks_client_get_job_description:
+ * @client: A #UDisksClient.
+ * @job: A #UDisksJob.
+ *
+ * Gets a human-readable and localized text string describing the
+ * operation of @job.
+ *
+ * For known job types, see the documentation for the
+ * <link linkend="gdbus-property-org-freedesktop-UDisks2-Job.Operation">Job:Operation</link>
+ * D-Bus property.
+ *
+ * Returns: (transfer full): A string that should be freed with g_free().
+ */
+gchar *
+udisks_client_get_job_description (UDisksClient   *client,
+                                   UDisksJob      *job)
+{
+  g_return_val_if_fail (UDISKS_IS_CLIENT (client), NULL);
+
+  return udisks_client_get_job_description_from_operation (udisks_job_get_operation (job));
+}
+
 /* ---------------------------------------------------------------------------------------------------- */
 
 static UDisksPartitionTypeInfo *
